/*****************************************************************************
 *
 * Main entry into the application.
 *
 *****************************************************************************
 * FileName:        main.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Calvin Ho			06/07/05	...	
 * 
 *****************************************************************************/

#include	<p30FXXXX.h>
#include 	<uart.h>
#include	<stdio.h>
#include 	<can.h>

#include "GLCD_30IO_APP023.h"
#include "i2c.h"

void	T1_Initial(void) ;
void	ADC_Initial(void) ;
void	AD_Delay(void) ;
void	PMP_Initial(void);
void	KEYS_Test(void) ;
void	IOS_Initial(void) ;
void	OFF_LED1to8(void) ;
void	OC1_Initial(void) ;
void	TMR2_Initial(void) ;
void	uitoa(unsigned int , unsigned char *);
void	Init_Mode0(void);
void	Initial_CAN( void );

#define	LED1	LATBbits.LATB12
#define	LED2	LATBbits.LATB13
#define	LED3	LATBbits.LATB14
#define	LED4	LATBbits.LATB15
#define	LED5	LATDbits.LATD12
#define	LED6	LATDbits.LATD13
#define	LED7	LATDbits.LATD14
#define	LED8	LATDbits.LATD15

#define	DIR_LED1	TRISBbits.TRISB12
#define	DIR_LED2	TRISBbits.TRISB13
#define	DIR_LED3	TRISBbits.TRISB14
#define	DIR_LED4	TRISBbits.TRISB15
#define	DIR_LED5	TRISDbits.TRISD12
#define	DIR_LED6	TRISDbits.TRISD13
#define	DIR_LED7	TRISDbits.TRISD14
#define	DIR_LED8	TRISDbits.TRISD15

#define	SW1		PORTEbits.RE9 
#define	SW2		PORTAbits.RA15
#define	DIR_SW1	TRISEbits.TRISE9
#define	DIR_SW2	TRISAbits.TRISA15

#define	ELED1	LATEbits.LATE6
#define	ELED2	LATEbits.LATE7

#define	DIR_ELED1		TRISEbits.TRISE6
#define	DIR_ELED2		TRISEbits.TRISE7

#define	ES2				PORTAbits.RA9
#define	ES3				PORTAbits.RA10

#define	DIR_ES2			TRISAbits.TRISA9
#define	DIR_ES3			TRISAbits.TRISA10

#define	DIR_BUZZER		TRISEbits.TRISE0 

#define	INPUT 			1
#define	OUTPUT			0

#define	LED_ACTIVE		0
#define	LED_INACTIVE	1

#define	EVR1			0b0000000000001000	//	AN8
#define	EVR2			0b0000000000001001  // 	AN9
#define	KEYVR			0b0000000000000010	//	AN2 for Key Input !!
#define	VR1				0b0000000000000110	//	AN6 for VR1
#define	VR2				0b0000000000000111	// 	AN7 for VR2 

#define					Ctrl_U1RX 	TRISFbits.TRISF2 
#define					Ctrl_U1TX 	TRISFbits.TRISF3 

int		ELED1_Loop = 0 ;
int		ELED2_Loop = 0 ;
int		AD1_Value , AD2_Value ;
int		AD1_Flash = 100 ;
int		AD2_Flash = 100 ;

int		ADC_TempValue ;
int		Buzzer_ON = 0 ;
int		Buzzer_Timer = 0 ;

int		VR1_Value , VR2_Value ;

int		T1_Counter = 0 ;
int		Update_LCD = 0 ;
int		Display_Mode = 0 ;

unsigned char	MCP9800_Value ;
unsigned char 	ASCII_Buffer[10] ;

unsigned char	TxData[10] = {0,0,0,0,0,0,0,0,0,0} ;

extern int 		__C30_UART ;

//
//		There are two VRs on APP023-4
//		EVR1 @ RB12
//		EVR2 @ RB13
//		Use EVR1 to control the flash rate of LED1
//		Use EVR2 to control the flash rate of LED2
//

void	_ISR	_T1Interrupt(void)
{
		IFS0bits.T1IF = 0 ;				//   M Timer1 _X 

		T1_Counter ++ ;
		if ( T1_Counter >= 300 )	
		{
			T1_Counter = 0 ;
			Update_LCD = 1 ;
		}
		
		ELED1_Loop ++ ;
		ELED2_Loop ++ ;

		if ( Buzzer_ON )
			 	{
					Buzzer_Timer ++ ;
					if (Buzzer_Timer > 20)	
						{
							Buzzer_ON = 0 ;
							Buzzer_Timer = 0 ;
							PTCONbits.PTEN = 0 ;
						}
				}
			

			if ( !ES2 )		
					{
						ELED1 = LED_INACTIVE ;
						LED7 = 0 ;					//	Turn ON LED7 ;
					}

			else if ( ELED1_Loop >= AD1_Flash )	
				{
					ELED1_Loop = 0 ;
					ELED1 = ~ ELED1 ;
					LED7 = 1 ;
				}
			
			if ( !ES3 )		
					{
						ELED2 = LED_INACTIVE ;
						LED8 = 0 ;
					}
			else if ( ELED2_Loop >= AD2_Flash )
				{
					ELED2_Loop = 0 ;
					ELED2 = ~ ELED2 ;
					LED8 = 1 ;
				}


}

void	MPWM_Initial(void)
{
		IEC2bits.PWMIE = 0 ;		// Disable PWM Interrupt !!
		IEC2bits.FLTAIE = 0 ;	

		OVDCON = 0xff00 ;			// Inactive all PWM OUTPUT !!

		TRISE &= 0b1111111111111110 ;		
		PTCON = 0x2008 ;		// Configure as 0b0010 0000 0000 1000 
								// PWM Time Base OFF , PWM Time Base OP in free running Mode 
		PWMCON1 = 0x0f01 ;		// Configure as 0b0000000000000001 , Only PWM1L as PWM
								// PWM I/O in complementary Mode and only PWM1L/H as PWM output 
		PWMCON2 = 0x0000 ;		// Configure as 0b0000000000000000 

		DTCON1 = 0x0101 ;		// Configure as 0b0000001000000010 ;

		FLTACON = 0x0000 ;

		IPC9bits.PWMIP = 6 ;

		PTPER = 250 ;			// PWM Time Base Period Register 

		PDC1 = 150 ;		

}

void	Initial_UART1(void)
{

		U1MODE = 0x8800 ;
		U1STA = 0x2400 ;
		U1BRG = 51 ;					// Baud-Rate = 19200 when Fcy = 4M , BRGH = 0 

		Ctrl_U1RX = INPUT ;
		Ctrl_U1TX = OUTPUT ;
	
}

int main(void)
{

		SRbits.IPL = 0 ;

		T1_Initial() ;
 
		IOS_Initial() ;
	
		ADC_Initial() ;

		TMR2_Initial() ;
		
		LCD_Delay200usX(100) ;

		LCD_Initial() ;

		I2C1_Initial() ;					// 	Initial I2C module 1 

		MPWM_Initial() ;

		Initial_UART1() ;

		Initial_CAN() ;						// Initial CAN to 125K bps

		LCD_Delay200usX(200) ;
		putrsLCD("ڬOPIC30F6010A");

		Init_Mode0() ;

			TxData[0] = 0 ;
			TxData[1] = 0 ;
			TxData[2] = 0 ;
			TxData[3] = 0 ;
			TxData[4] = 0 ;
			TxData[5] = 0 ;
			TxData[6] = 0 ;
			TxData[7] = 0 ;

	while(1) 
	{


		ADCHS = EVR1 ;
		AD_Delay() ;
		ADCON1bits.SAMP = 0 ;			//	Start Conversion 
		Nop() ;
		Nop() ;
		while ( ! ADCON1bits.DONE ) ;
		AD1_Value = ADCBUF0 ;			// 	0 .. 1023
		AD1_Flash = 100 + AD1_Value/3 ;	



		ADCHS= EVR2 ;
		AD_Delay() ;
		ADCON1bits.SAMP = 0 ;			//	Start Conversion 
		Nop() ;
		Nop() ;
		while ( ! ADCON1bits.DONE ) ;
		AD2_Value = ADCBUF0 ;			// 	0 .. 1023
		AD2_Flash = 100 + AD2_Value/3 ;		

		ADCHS = VR1 ;
		AD_Delay() ;
		ADCON1bits.SAMP = 0 ;			//	Start Conversion 
		Nop() ;
		Nop() ;
		while ( ! ADCON1bits.DONE ) ;
		VR1_Value = ADCBUF0 ;			// 	0 .. 1023


		ADCHS = VR2 ;
		AD_Delay() ;
		ADCON1bits.SAMP = 0 ;			//	Start Conversion 
		Nop() ;
		Nop() ;
		while ( ! ADCON1bits.DONE ) ;
		VR2_Value = ADCBUF0 ;			// 	0 .. 1023

			TxData[0] = 0 ;
			TxData[1] = 0 ;
			TxData[2] = 0 ;
			TxData[3] = (char)((VR1_Value>>8) & 0x00ff) ;
			TxData[4] = (char)(VR1_Value & 0x00ff) ;


			if ( Update_LCD == 1 )
				{
					Update_LCD = 0 ;
					setcurLCD(3,2) ;
					putrsLCD("    ") ;
					uitoa( VR1_Value , ASCII_Buffer ) ;
					setcurLCD(3,2) ;
					putsLCD(ASCII_Buffer) ;
					setcurLCD(3,3) ;
					putrsLCD("    ") ;
					uitoa( VR2_Value , ASCII_Buffer ) ;
					setcurLCD(3,3) ;
					putsLCD(ASCII_Buffer) ;

					MCP9800_Value = ReadI2C1wAddr(0x9a);
					uitoa((unsigned)MCP9800_Value,ASCII_Buffer) ;

					setcurLCD(3,1) ;
					putsLCD(ASCII_Buffer);

					printf("I am dsPIC30F6010A \n\r") ;

					CAN1SendMessage( 	( CAN_TX_SID(0x200)) & CAN_TX_EID_DIS & CAN_SUB_NOR_TX_REQ , 
										( CAN_TX_EID(0x00)) & CAN_NOR_TX_REQ , TxData , 8 , 0 ) ;

				}

		KEYS_Test() ;					// 	Test SW1 .. SW6 and LED1 .. LED6 

	}

}

void	Init_Mode0(void)
{
		setcurLCD(0,1) ;
		putrsLCD("ū :          ") ;
		setcurLCD(0,2) ;
		putrsLCD("q1:          ") ;
		setcurLCD(0,3) ;
		putrsLCD("q2:          ") ;
}
		

void 	IOS_Initial(void)
{
	DIR_ELED1 = OUTPUT ;
	DIR_ELED2 = OUTPUT ;

	DIR_ES2 = INPUT ;
	DIR_ES3 = INPUT ;

	DIR_SW1 = INPUT ;
	DIR_SW2 = INPUT ;

	ELED1 = 0 ;
	ELED2 = 1 ;

	LED1 = 1 ;
	LED2 = 1 ;
	LED3 = 1 ;
	LED4 = 1 ;
	DIR_LED1 = OUTPUT ;
	DIR_LED2 = OUTPUT ;
	DIR_LED3 = OUTPUT ;
	DIR_LED4 = OUTPUT ;

	LED5 = 1 ;
	LED6 = 1 ;
	LED7 = 1 ;
	LED8 = 1 ;
	DIR_LED5 = OUTPUT ;
	DIR_LED6 = OUTPUT ;
	DIR_LED7 = OUTPUT ;
	DIR_LED8 = OUTPUT ;

	DIR_BUZZER = INPUT ;

}

void	OFF_LED1to6(void)
{
	LED1 = 1 ;
	LED2 = 1 ;
	LED3 = 1 ;
	LED4 = 1 ;
	LED5 = 1 ;
	LED6 = 1 ;

}

void 	KEYS_Test(void)
{
		int	KEY_Index  ;

		OFF_LED1to6() ;
		ADCHS = KEYVR  ;
		AD_Delay() ;
		ADCON1bits.SAMP = 0 ;			//	Start Conversion 
		Nop() ;
		Nop() ;
		while ( ! ADCON1bits.DONE ) ;

		ADC_TempValue = ADCBUF0 ;
		
		KEY_Index = 0 ;

			if 		( ADC_TempValue > 680 && ADC_TempValue < 690 )	KEY_Index = 6 ;
			else if	( ADC_TempValue > 500 && ADC_TempValue < 520 )  KEY_Index = 5 ;
			else if ( ADC_TempValue > 340 && ADC_TempValue < 350 )  KEY_Index = 4 ;
			else if ( ADC_TempValue == 0 )							KEY_Index = 3 ;

		if ( ! SW1 ) 	KEY_Index = 1 ;
		if ( ! SW2 ) 	KEY_Index = 2 ;

		switch ( KEY_Index ) 
		{
			case 6 :
					LED6 = 0 ;
					break ;
			case 5 : 
					LED5 = 0 ;
					break ;
			case 4 :
					LED4 = 0 ;
					break ;
			case 3 :
					LED3 = 0 ;
					break ;
			case 2 :
					LED2 = 0 ;
					break ;
			case 1 :
					LED1 = 0 ;
					break ;

		}

			if ( KEY_Index != 0 )		
				{
					Buzzer_ON = 1 ;
					Buzzer_Timer = 0 ;
					PTCONbits.PTEN = 1 ;
				}
}

void	AD_Delay(void)
{
		int	LoopVar ;
		int	LoopVar2 , LoopVar3 ;
		
		for ( LoopVar = 0 ; LoopVar < 100 ; LoopVar++ )
			LoopVar2 = LoopVar3 + 1 ;
		
}


void	TMR2_Initial(void)
{
		T2CON = 0b1000000000000000 ;
}
void	T1_Initial(void)
{
	PR1 = 10000 ;
	T1CON = 0b0000000000000000 ;
	T1CONbits.TON = 1 ;
	IEC0bits.T1IE = 1 ;
}

void	ADC_Initial(void)
{
		ADCON1 = 0b0000000000000110 ;	// 	Mode : 	Clear SAMP start conversion
										//		   	ASAM = 1 --> SAMP auto set 
		ADCON2 = 0b0000000000000000 ;
		ADCON3 = 0b1000001100001111 ;	//	Internal RC as AD Clock 
		ADCHS = 0x00 ;					//  Use AN0 temporary 
		ADPCFG = 0b1111110000011011 ;	// 	AN8 & AN9 , AN0.. AN2 enabled
		ADCON1bits.ADON = 1 ;

}


void	uitoa(unsigned int Input_I , unsigned char *Output_A)
{
		int		Temp_Result ;
		int		Zero_Fill ;

		Zero_Fill = 0 ;
		Temp_Result = Input_I / 10000 ;
		if ( Temp_Result != 0 )
			{
				*Output_A++ = Temp_Result + '0' ;
				Zero_Fill = 1 ;
			}
		Input_I = Input_I - ( Temp_Result * 10000 ) ;

		Temp_Result = Input_I / 1000 ;
		if ( Temp_Result != 0 )
			{
				*Output_A++ = Temp_Result + '0' ;
				Zero_Fill = 1 ;
			}
		else if ( Zero_Fill )	*Output_A++ = '0' ;
		Input_I = Input_I - ( Temp_Result * 1000 ) ;	

		Temp_Result = Input_I / 100 ;
		if ( Temp_Result != 0 )
			{
				*Output_A++ = Temp_Result + '0' ;
				Zero_Fill = 1 ;
			}
		else if ( Zero_Fill )	*Output_A++ = '0' ;
		Input_I = Input_I - ( Temp_Result * 100 ) ;	

		Temp_Result = Input_I / 10 ;
		if ( Temp_Result != 0 )
			{
				*Output_A++ = Temp_Result + '0' ;
				Zero_Fill = 1 ;
			}
		else if ( Zero_Fill )	*Output_A++ = '0' ;
		Input_I = Input_I - ( Temp_Result * 10 ) ;	

		*Output_A++ = Input_I + '0' ;
		*Output_A = 0x00 ;
}
