/*****************************************************************************
 *
 * LCD Driver for PIC33.
 *
 *****************************************************************************
 * FileName:        lcd.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * A simple LCD driver for LCDs interface through the PMP
 * 
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Ross Fosler			08/13/04	...	
 
 *****************************************************************************/

#include	<p30FXXXX.h>
#include	"GLCD_30IO_APP023.h"

#define		CPU_FCY		16

#define		LCD_RS		LATBbits.LATB10		//	RS = 1 >> Data , =0 >> Command
#define		LCD_RW		LATBbits.LATB11		//  RW = 0 >> Write , = 1 >> Read
#define		LCD_E		LATGbits.LATG9

#define		DIR_LCD_RS	TRISBbits.TRISB10
#define		DIR_LCD_RW	TRISBbits.TRISB11
#define		DIR_LCD_E	TRISGbits.TRISG9

unsigned char	LCD_StatusByte ;

int	LCDIsBusy(void)
{
	int	BusyLoop ;

	for (BusyLoop = 0 ; BusyLoop < 30 ; BusyLoop++) ;

		LCD_StatusByte = cmdreadLCD() ;

	for (BusyLoop = 0 ; BusyLoop < 30 ; BusyLoop++) ;

			return ((LCD_StatusByte >> 7) & 0x0001) ;
}

void	LCD_Initial(void)
{
		TRISD &= 0b1111011110000000 ;

		LCD_E = 0 ;

		DIR_LCD_RS = 0 ;
		DIR_LCD_RW = 0 ;
		DIR_LCD_E = 0 ;
		
		LCD_E_Delay() ;

		cmdwriteLCD(0x00) ;
		LCD_Delay200usX(40) ;
		cmdwriteLCD(0x30) ;
		LCD_Delay200usX(25);			// Wait for > 4.1 ms --> 200us * 25 = 5 ms
		cmdwriteLCD(0x30) ;
		LCD_Delay200usX(20) ;
		cmdwriteLCD(0x30) ;
		LCD_Delay200usX(10) ;
		cmdwriteLCD(0x08) ;
		LCD_Delay200usX(10) ;
		cmdwriteLCD(0x01) ;
		LCD_Delay200usX(50) ;				// Display OFF
		cmdwriteLCD(0x06) ;
		LCD_Delay200usX(10) ;			// Clear Display = 1.5 ms 
		cmdwriteLCD(0x30) ;
		LCD_Delay200usX(10) ;
		cmdwriteLCD(0x08) ;					// Display ON 
		LCD_Delay200usX(10) ;
		cmdwriteLCD(0x0e) ;
		LCD_Delay200usX(10) ;
		cmdwriteLCD(0x06) ;					// Display ON 
		LCD_Delay200usX(50) ;
}

void	setcurLCD( char Cur_X , char Cur_Y )
{
 
		unsigned char	BaseAddr ;

		switch ( Cur_Y )
		{
			case	0 :
					BaseAddr = 0x80 ;
					break ;
			case	1 :
					BaseAddr = 0x90 ;
					break ;
			case	2 :
					BaseAddr = 0x88 ;
					break ;
			case	3 :
					BaseAddr = 0x98 ;
					break ;
			default :
					BaseAddr = 0x80 ;
					break ;
		}
			BaseAddr += Cur_X ;
			while ( LCDIsBusy()) ;
			cmdwriteLCD(BaseAddr) ;

}


void 	putcLCD( unsigned char TheChar)
{
			datawriteLCD((unsigned int)TheChar);
}

void	putsLCD( unsigned char *OutString )
{
		while ( *OutString != 0x00 )
		{
 			while ( LCDIsBusy() );
			putcLCD( *OutString++ ) ;
		}
}

void	putrsLCD( const unsigned char *OutString )
{
		while ( *OutString != 0x00 )
		{
 			while ( LCDIsBusy() );
			putcLCD( *OutString++ ) ;
		}
}

void 	LCD_Delay200usX(int	UnitOf200us)
{
		int		Loop_200usX ;

		for ( Loop_200usX = 0 ; Loop_200usX < UnitOf200us ; Loop_200usX ++ )
		LCD_Delay200us( ) ;
}

void 	LCD_Delay200us(void)
{
	int		Loop_200us ;

	for ( Loop_200us = 0 ; Loop_200us < CPU_FCY * 40  ; Loop_200us++)	;

}


//		LCD Data @ RD0 .. RD6 and RD10
unsigned char	datareadLCD(void)				//	Read data from LCD
{
		int				Temp_PORTD ;
		int				Return_Data ;
		TRISD |= 0b0000100001111111 ;			// Make all LCD data pins as INPUT 
		LCD_RS = 1 ;
		LCD_RW = 1 ;
		LCD_E_Delay() ;
		LCD_E = 1 ;
		LCD_E_Delay() ;
		Temp_PORTD = PORTD ;
		LCD_E = 0 ;
		TRISD &= 0b1111011110000000 ;
		Return_Data = Temp_PORTD ;
		if ( Temp_PORTD & 0b0000100000000000 ) 		Return_Data |= 0b10000000 ;
		else										Return_Data &= 0b01111111 ;

		return 	(unsigned char)Return_Data ;
}

unsigned char	cmdreadLCD(void)				//	Read data from LCD
{
		int				Temp_PORTD ;
		int				Return_Data ;

		TRISD |= 0b0000100001111111 ;			// Make all LCD data pins as INPUT 
		LCD_RS = 0 ;
		LCD_RW = 1 ;
		LCD_E_Delay() ;
		LCD_E = 1 ;
		LCD_E_Delay() ;
		Temp_PORTD = PORTD ;
		LCD_E = 0 ;
		TRISD &= 0b1111011110000000 ;
		Return_Data = Temp_PORTD ;
		if ( Temp_PORTD & 0b0000100000000000 ) 		Return_Data |= 0b10000000 ;
		else										Return_Data &= 0b01111111 ;

		return	(unsigned char)Return_Data ;
}

void	datawriteLCD(unsigned char wdata)		// Write data to LCD
{
		LATD &= 0b1111011110000000 ;
		if ( wdata & 0b0000000010000000 )
			LATD |=  0b0000100000000000 ;
			wdata &= 0b01111111 ;
		LATD |= wdata ;
		LCD_RS = 1 ;
		LCD_RW = 0 ;
		LCD_E_Delay() ;
		LCD_E = 1 ;
		LCD_E_Delay() ;
		LCD_E = 0 ;
		LCD_E_Delay() ;
}

void	cmdwriteLCD(unsigned char wdata)		// Write Command to LCD 
{
		LATD &= 0b1111011110000000 ;
		if ( wdata & 0b0000000010000000 )
			LATD |=  0b0000100000000000 ;
			wdata &= 0b01111111 ;
		LATD |= wdata ;
		LCD_RS = 0 ;
		LCD_RW = 0 ;
		LCD_E_Delay() ;
		LCD_E = 1 ;
		LCD_E_Delay() ;
		LCD_E = 0 ;
		LCD_E_Delay() ;
}

void	LCD_E_Delay(void)
{
		int	E_Loop ;
		for ( E_Loop = 0 ; E_Loop < CPU_FCY * 5 ; E_Loop ++ ) ;
}


