/*****************************************************************************
 *
 * Main entry into the application.
 *
 *****************************************************************************
 * FileName:        main.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Calvin Ho			06/07/05	...	
 * 
 *****************************************************************************/

#include	<p24FXXXX.h>

#include "GLCD_APP023.h"

void	T1_Initial(void) ;
void	ADC_Initial(void) ;
void	AD_Delay(void) ;
void	PMP_Initial(void);

#define	LED1	LATAbits.LATA9
#define	LED2	LATAbits.LATA10

#define	DIR_LED1		TRISAbits.TRISA9
#define	DIR_LED2		TRISAbits.TRISA10

#define	ES2				PORTAbits.RA14
#define	ES3				PORTAbits.RA15

#define	DIR_ES2			TRISAbits.TRISA14
#define	DIR_ES3			TRISAbits.TRISA15

#define	INPUT 			1
#define	OUTPUT			0

#define	LED_ACTIVE		0
#define	LED_INACTIVE	1

#define	EVR1			0b0000000000001100	//	AN12
#define	EVR2			0b0000000000001101  // 	AN13

int		LED1_Loop = 0 ;
int		LED2_Loop = 0 ;
int		AD1_Value , AD2_Value ;
int		AD1_Flash = 100 ;
int		AD2_Flash = 100 ;

//
//		There are two VRs on APP023-4
//		EVR1 @ RB12
//		EVR2 @ RB13
//		Use EVR1 to control the flash rate of LED1
//		Use EVR2 to control the flash rate of LED2
//

void	_ISR	_T1Interrupt(void)
{
		IFS0bits.T1IF = 0 ;				//   M Timer1 _X 
		
		LED1_Loop ++ ;
		LED2_Loop ++ ;

			if ( !ES2 )		LED1 = LED_INACTIVE ;

			else if ( LED1_Loop >= AD1_Flash )	
				{
					LED1_Loop = 0 ;
					LED1 = ~ LED1 ;
				}
			
			if ( !ES3 )		LED2 = LED_INACTIVE ;

			else if ( LED2_Loop >= AD2_Flash )
				{
					LED2_Loop = 0 ;
					LED2 = ~ LED2 ;
				}
}

int main(void)
{
	T1_Initial() ;
	ADC_Initial() ;
	PMP_Initial() ;
 
	DIR_LED1 = OUTPUT ;
	DIR_LED2 = OUTPUT ;

	DIR_ES2 = INPUT ;
	DIR_ES3 = INPUT ;

	LED1 = 0 ;
	LED2 = 1 ;
		
		LCD_Delay200usX(100) ;

		LCD_Initial() ;

		LCD_Delay200usX(200) ;
		putsLCD("I am PIC24FJ128");
	
	while(1) 
	{

		AD1CHS = EVR1 ;
		AD_Delay() ;
		AD1CON1bits.SAMP = 0 ;			//	Start Conversion 
		Nop() ;
		Nop() ;
		while ( ! AD1CON1bits.DONE ) ;
		AD1_Value = ADC1BUF0 ;			// 	0 .. 1023
		AD1_Flash = 100 + AD1_Value/3 ;	


		AD1CHS = EVR2 ;
		AD_Delay() ;
		AD1CON1bits.SAMP = 0 ;			//	Start Conversion 
		Nop() ;
		Nop() ;
		while ( ! AD1CON1bits.DONE ) ;
		AD2_Value = ADC1BUF0 ;			// 	0 .. 1023
		AD2_Flash = 100 + AD2_Value/3 ;			
	}

	Nop() ;
}

void	AD_Delay(void)
{
		int	LoopVar ;
		
		for ( LoopVar = 0 ; LoopVar < 100 ; LoopVar++) ;
}

void	T1_Initial(void)
{
	PR1 = 4000 ;
	T1CON = 0b0000000000000000 ;
	T1CONbits.TON = 1 ;
	IEC0bits.T1IE = 1 ;
}

void	ADC_Initial(void)
{
		AD1CON1 = 0b0000000000000110 ;	// 	Mode : 	Clear SAMP start conversion
										//		   	ASAM = 1 --> SAMP auto set 
		AD1CON2 = 0b0000000000000000 ;
		AD1CON3 = 0b1000001100001111 ;	//	Internal RC as AD Clock 
		AD1CHS = 0x00 ;					//  Use AN0 temporary 
		AD1PCFG = 0b1100111111111111 ;	// 	AN12 & AN13 enabled
		AD1CON1bits.ADON = 1 ;
}

void	PMP_Initial(void)
{
			//********************************************************************************
			//  STEP 1:
			//	Configure PMPCON: PMP on, address/data not multiplexed, PMPBE active high, 
			//	PMPWR I/O, PMPRD I/O, 8-bit data, PMPENB and PMPRD/~PMPWR active high.
			//********************************************************************************/
			PMCON = 0b1000001100000111;


			//********************************************************************************
			//	STEP 2:
			//	Configure PMPMODE: Interrupts, stall, buffers, inc/dec off, 8 bit mode,
			//	combined read/write with byte enable signals, and max the 3 wait delays.
			//********************************************************************************/
			PMMODE = 0x03FF;

			//********************************************************************************
			//	STEP 3:	
			//	Configure PMPEN: Enable A0 function to control RS and disable all other 
			//	PMP address pins.
			//********************************************************************************/
			PMPEN = 0x0001;

			//********************************************************************************
			// 	STEP 4:
			//	Configure PMPADDR: A0 selects type of instruction, either command or data.
			//	This is a command so A0 should be low.
			//********************************************************************************/
			PMADDR = 0x0000;
}
