/*****************************************************************************
 *
 * LCD Driver for PIC24.
 *
 *****************************************************************************
 * FileName:        lcd.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * A simple LCD driver for LCDs interface through the PMP
 * 
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Ross Fosler			08/13/04	...	
 
 *****************************************************************************/

#include	<p24FXXXX.h>
#include	"GLCD_APP023.h"

#define		LCD_CMD_W 0x0000
#define		LCD_CMD_R 0x0000
#define		LCD_DAT_W 0x0001

int	LCDIsBusy(void)
{
	int	BusyLoop ;
	unsigned char	LCD_StatusByte ;

	for (BusyLoop = 0 ; BusyLoop < 20 ; BusyLoop++) ;

			PMADDR = LCD_CMD_R ;

			LCD_StatusByte = PMDIN1;

	for (BusyLoop = 0 ; BusyLoop < 20 ; BusyLoop++) ;

			return ((LCD_StatusByte >> 7) & 0x0001) ;
}

void	LCD_Initial(void)
{

		PMADDR = LCD_CMD_W ;
		PMDIN1 = 0x00 ;
		LCD_Delay200usX(40) ;
		PMDIN1 = 0x30 ;
		LCD_Delay200usX(25);			// Wait for > 4.1 ms --> 200us * 25 = 5 ms
		PMDIN1 = 0x30 ;
		LCD_Delay200usX(20) ;
		PMDIN1 = 0x30 ;
		LCD_Delay200usX(10) ;
		PMDIN1 = 0x08 ;
		LCD_Delay200usX(10) ;
		PMDIN1 = 0x01 ;
		LCD_Delay200usX(50) ;				// Display OFF
		PMDIN1 = 0x06 ;
		LCD_Delay200usX(10) ;			// Clear Display = 1.5 ms 
		PMDIN1 = 0x30 ;
		LCD_Delay200usX(10) ;
		PMDIN1 = 0x08 ;					// Display ON 
		LCD_Delay200usX(10) ;
		PMDIN1 = 0x0e ;
		LCD_Delay200usX(10) ;
		PMDIN1 = 0x06 ;					// Display ON 
		LCD_Delay200usX(50) ;
}

void	LCD_SetCursor( char Cur_X , char Cur_Y )
{
		char	Cur_Location ;

		Cur_Location = 0x80 + (Cur_Y*0x40) + Cur_X ;

		while(LCDIsBusy()) ;
		PMADDR = LCD_CMD_W ;
		PMDIN1 = Cur_Location ; 

}

void LCD_PutChar( unsigned char TheChar)
{

			PMADDR = LCD_DAT_W;
			PMDIN1 = (unsigned int)TheChar;
}

void 	putcLCD( unsigned char TheChar)
{

			PMADDR = LCD_DAT_W;
			PMDIN1 = (unsigned int)TheChar;
}

void	putsLCD( unsigned char *OutString )
{
		while ( *OutString != 0x00 )
		{
 			while ( LCDIsBusy() );
			putcLCD( *OutString++ ) ;
		}
}

void 	LCD_Delay200usX(int	UnitOf200us)
{
		int		Loop_200usX ;

		for ( Loop_200usX = 0 ; Loop_200usX < UnitOf200us ; Loop_200usX ++ )
		LCD_Delay200us( ) ;
}

void 	LCD_Delay200us(void)
{
	int		Loop_200us ;

	for ( Loop_200us = 0 ; Loop_200us < 130 ; Loop_200us++)	;

}
