

#include <p18f8680.h>	// p18cxxx.h must have current processor
#include "CAN18XX8.h"	// Purpose of using AN738 Subroutine !!
#include "APP023_GLCD.h"
#include <stdlib.h>
#include <usart.h>
#include <string.h>
#include <delays.h>
#include "vt100.h"

#define	MAX_ID_SUPPORTED  	10
#define	SW1						PORTBbits.RB0
#define	SW2						PORTAbits.RA4
#define	TRIS_SW1					TRISBbits.TRISB0 
#define	TRIS_SW2					TRISAbits.TRISA4	
#define	LED_PORT 				LATE
#define	TRIS_LED_PORT			TRISE 

#define	INPUT		1 
#define	OUTPUT	0
	
void 	Delay_100(void) ;
void	hexbtoa(unsigned char , unsigned char *) ;
void	PrintRXData( void ) ;
void	CAN_Monitor_Init( void ) ;

char 		TempASCII[10] ;

char		OutBuffer1[64] ;
int			ID_Buffer[MAX_ID_SUPPORTED] ;
char		SW1_LOCKED = 0 ;
char		SW1_Counter ; 
char		Y_PositionNow ;
unsigned	PrintLoop ;

int			SendLoop = 0  ;

const rom unsigned long	My_IDs[8] = { 0x000200 , 0x000201 , 0x000202 , 0x000203 , 0x000204 , 0x000205 , 0x000206 , 0x000207 } ;

const rom unsigned char Welcome1[] = "    wѥ[  " ;
const rom unsigned char Welcome2[] = " ECAN Workshop" ;

int	PrintCounter = 0 , NullCounter = 0  , PacketNumber = 0 ;

union
	{
		unsigned char	Bytes[2] ;
		unsigned int		Word ;	
	}     Word2Byte  ;

	unsigned char 	Var1,Var2 ;

	unsigned long	TX_ID1 ;
	BYTE			TX_Data_Buf1[8] ;

	unsigned long	RX_ID1 ;
	BYTE			RX_Data_Buf1[8] ;
	BYTE			RX_Data_Len1	;
	enum 			CAN_RX_MSG_FLAGS	RX1_Message_Flag ; 	

	unsigned long	RX_ID2 ;
	BYTE			RX_Data_Buf2[8] ;
	BYTE			RX_Data_Len2	;
	enum 			CAN_RX_MSG_FLAGS	RX2_Message_Flag ; 			

#define			MESSAGE_ID1			0x200
#define			RX_Filter0				0x100
#define			RX_Filter1				0x00
#define			RX_Filter2				0x00
#define			RX_Filter3				0x0
#define			RX_Filter4				0x00
#define			RX_Filter5				0x00
#define			RXB0_MASK			0x7ff		// 0b111 1111 1100
#define			RXB1_MASK			0x7ff

void InitializeUSART(void)
{
    	TRISCbits.TRISC7=1 ; // RX
    	TRISCbits.TRISC6=1 ; // TX

    	SPBRG = 0x08 ;
    	SPBRGH = 0x02;      	// 0x208 for 19200 bps @ 40Mhz Fosc
							// Formula : BaudRate = Fosc/4(BRG+1)  if BRG16 = 1 and BRGH = 1 ;
							// Thus ..      BRG = (Fosc/(4*BaudRate)) - 1 

    	TXSTA = 0x24;       		// TX enable BRGH=1
    	RCSTA = 0x90;       		// continuous RX
    	BAUDCON = 0x08;     	// BRG16 = 1

}//end InitializeUSART

void main( void )
{
	int 	Loop1 , Loop2 ;

	TRISGbits.TRISG0 = 0 ;				// CANTX
	TRISGbits.TRISG2 = 1 ;				// CANRX

	TRIS_SW1	= INPUT ;
	TRIS_SW2 = INPUT ;
	TRIS_LED_PORT = OUTPUT ;
	
	//	***************************************************************									
	//	AD Modul initialize for PIC18F458X
	//	***************************************************************
	ADCON1 = 	0b00001110 ;
	ADCON2 = 	0b10100010 ;			//  	Right Justified !!
	ADCON0 = 	0b00000001 ;			// 	Initial ADCON0 for A/D Module 
	CMCON = 0b00000111 ;			// 	Disable Comparator module !!

		LCD_Delay_nms( 200 ) ;

		GLCD_Initial( ) ;

		 InitializeUSART( ) ;

				setcurGLCD(0,0) ;
				putrsGLCD( Welcome1 ) ;
	
				setcurGLCD(1,0) ;				// Set Cursor @ ( Y,X )

				putrsGLCD( Welcome2 ) ;

				setcurGLCD(2,0) ;
				putrsGLCD("Packet TX:") ;
				setcurGLCD(3,0) ;
				putrsGLCD("Packet RX:") ;
			

				InitializeVT100( ) ;

				Delay_100( ) ;

				CAN_Monitor_Init( ) ;
	
	//
	// Initialize CAN Communication Module !!
	// The Sequence of Parameter is SJW,BRP,PHASEG1,PHASEG2,PROPSEG, Mode !!
	//
	
	CANInitialize(   2,10,6,6,3,	       CAN_CONFIG_LINE_FILTER_OFF & 
									CAN_CONFIG_SAMPLE_ONCE & 
									CAN_CONFIG_VALID_STD_MSG &
									CAN_CONFIG_PHSEG2_PRG_ON ) ;

	CANSetOperationMode(CAN_OP_MODE_CONFIG) ;

	CANSetMask(CAN_MASK_B1, RXB0_MASK, CAN_CONFIG_STD_MSG ) ;	// Set Mask For Standard Data Frame 
	CANSetMask(CAN_MASK_B2, RXB1_MASK, CAN_CONFIG_STD_MSG ) ;
	
	CANSetFilter(CAN_FILTER_B1_F1, RX_Filter0  , CAN_CONFIG_STD_MSG) ;
	CANSetFilter(CAN_FILTER_B1_F2, RX_Filter1  , CAN_CONFIG_STD_MSG) ;
	CANSetFilter(CAN_FILTER_B2_F1,RX_Filter2 , CAN_CONFIG_STD_MSG) ;
	CANSetFilter(CAN_FILTER_B2_F2,RX_Filter3  , CAN_CONFIG_STD_MSG) ;
	CANSetFilter(CAN_FILTER_B2_F3,RX_Filter4  , CAN_CONFIG_STD_MSG) ;
	CANSetFilter(CAN_FILTER_B2_F4, RX_Filter5  , CAN_CONFIG_STD_MSG) ;

	CANSetOperationMode(CAN_OP_MODE_NORMAL ) ;

	TX_Data_Buf1[0] = 0x01 ;
	TX_Data_Buf1[1] = 0x02 ;
	TX_Data_Buf1[2] = 0x03 ;
	TX_Data_Buf1[3] = 0x04 ;
	TX_Data_Buf1[4] = 0x05 ;

	SW1_Counter = 0 ;

	LATE =  ( (char) PrintLoop ) ^ 0xff ;

	while (1)
	{
		NullCounter ++ ;

		if ( NullCounter > 500 )
		{
			NullCounter = 0 ;
			ADCON0bits.GO = 1 ;
		
				while ( ADCON0bits.GO) ;
		
				TX_Data_Buf1[3]  = ADRESH ;
				TX_Data_Buf1[4]  =  ADRESL ;
				
				if (CANIsTxReady( ) )

			{
				CANSendMessage(	MESSAGE_ID1,TX_Data_Buf1,0,CAN_TX_PRIORITY_0 &
									CAN_TX_STD_FRAME &
									CAN_TX_RTR_FRAME ) ;

				PacketNumber ++ ;

				setcurGLCD(2,5) ;						//  setcurGLCD( Y,X )
				putrsGLCD("      ") ;
				setcurGLCD(2,5) ;
				itoa ( PacketNumber , TempASCII ) ;
				putsGLCD(TempASCII ) ;
			}

		}




		if ( ! SW1 && ! SW1_LOCKED)
		{
			if ( SW1_Counter < 20 )
			{
				SW1_Counter ++ ;

				if ( SW1_Counter ==  20 )	
					{
						CAN_Monitor_Init( ) ;
						SW1_LOCKED = 1  ;
						Delay_100( ) ;
					}
			}			
		}

		else if ( SW1 && SW1_LOCKED )
			{
				SW1_Counter--  ;
				if ( SW1_Counter ==0 ) 	SW1_LOCKED = 0 ;
			}

		if ( CANIsRxReady( ) )

		{
			CANReceiveMessage(&RX_ID1,RX_Data_Buf1,&RX_Data_Len1,&RX1_Message_Flag ) ;


			Word2Byte.Bytes[0] = RX_Data_Buf1[4] ;
			Word2Byte.Bytes[1] = RX_Data_Buf1[3] ;


			for ( Loop1 = 0 ; Loop1 < MAX_ID_SUPPORTED ; Loop1 ++ ) 
				{
					
					if ( ((int)RX_ID1 != ID_Buffer[Loop1])  && (ID_Buffer[Loop1]  == 0 )) 
						{	
								ID_Buffer[Loop1] = (int)RX_ID1 ;
								VT100_Cursor_Position(Loop1+5,0) ;
								Y_PositionNow = Loop1 + 5 ;
								break ;
						}
					else if ( (int)RX_ID1 == ID_Buffer[Loop1] )  			
							{
								VT100_Cursor_Position ( Loop1 + 5 ,0 ) ;
								Y_PositionNow = Loop1 + 5 ;
								break ;
							}
				}
					PrintCounter ++ ;
					PrintRXData( ) ;

							putsUSART(OutBuffer1) ;
							VT100_Cursor_Position (Y_PositionNow , 37 ) ;
							VT100_Clear_Right( ) ;
							itoa ( PrintLoop , TempASCII ) ;
							putsUSART(TempASCII) ;					
							putrsUSART("\n\r") ;

							PrintLoop ++ ;
							if ( PrintLoop > 10000 )	PrintLoop = 0 ;

					setcurGLCD(3,5) ;
					putrsGLCD("      ") ;
					setcurGLCD(3,5) ;
					itoa ( PrintLoop , TempASCII ) ;
					putsGLCD(TempASCII ) ;

							LATE =  ( (char) PrintLoop ) ^ 0xff ;
		}
		
	}
	
}

void	CAN_Monitor_Init( void)
{
		int	Loop1 ;
		
		InitializeVT100( ) ;

			for ( Loop1 = 0 ; Loop1 < MAX_ID_SUPPORTED ; Loop1 ++ ) 
				{
							ID_Buffer[Loop1] = 0 ;		
				}

		VT100_Cursor_Position ( 1,0 ) ;
		putrsUSART( "PIC18F8680 Easy CAN Monitor \n\r ") ;
		VT100_Cursor_Position ( 3,0 ) ;
		putrsUSART("  ID    D0 D1 D2 D3 D4 D5 D6 D7     TIME") ;
		VT100_Cursor_Position ( 4,0 ) ;
		putrsUSART("-----------------------------------------") ;

		PrintLoop = 0 ;

}

void	PrintRXData( void )
{
		char	PrintRXDataLoop ;

						OutBuffer1[0] = 0x00 ;
						strcatpgm2ram(OutBuffer1 , "0x") ;
		
				Word2Byte.Word = RX_ID1 ;
				hexbtoa( Word2Byte.Bytes[1] , TempASCII ) ;
				strcat ( OutBuffer1 , TempASCII ) ;
				hexbtoa( Word2Byte.Bytes[0] , TempASCII) ;
				strcat ( OutBuffer1 , TempASCII ) ;				
	
						strcatpgm2ram(OutBuffer1 , "  ") ;

				for ( PrintRXDataLoop = 0 ; PrintRXDataLoop < RX_Data_Len1 ; PrintRXDataLoop ++ )
				{
							hexbtoa ( RX_Data_Buf1[PrintRXDataLoop] , TempASCII ) ;
							strcat (OutBuffer1 , TempASCII ) ;
							strcatpgm2ram(OutBuffer1 , " ") ;
				}

}

void	hexbtoa(unsigned char BinaryValue , unsigned char *ResultBuffer) 
{

		unsigned char TempChar1 ;
		
		TempChar1 = ( BinaryValue >> 4 ) & 0x0f ;
		if ( TempChar1 > 9 )		*ResultBuffer++ = TempChar1 + 'A'-10 ;
		else						*ResultBuffer++ = TempChar1 + '0' ;

		TempChar1 = ( BinaryValue & 0x0f ) ;
		if ( TempChar1 > 9 )		*ResultBuffer++ = TempChar1 + 'A'-10 ;
		else						*ResultBuffer++ = TempChar1 + '0' ;
		
		*ResultBuffer = 0x00 ;
}


void Delay_100(void)
{
	int  X1 ;

	for (X1 = 0 ; X1 < 10000 ; X1 ++ ) ;

}


//************************************************
//*       #pragma Interrupt Declarations         *
//************************************************
#pragma interrupt isr_high 		//	save = PRODL,PRODH,FSR0L,FSR0H
#pragma interruptlow isr_low 		//	save = WREG,STATUS,BSR,PRODL,PRODH,FSR0L,FSR0H

//************************************************
//* Function: isr_high_direct                    *
//*   - Direct execution to the actual           *
//*     high-priority interrupt code.            *
//************************************************
#pragma code isrhighcode = 0x0000008

void isr_high_direct(void)	
{  
   //***EXERCISE 2 CODE HERE***
                         										
     _asm 		//begin in-line assembly
     goto isr_high	//go to isr_high function
     _endasm 		//end in-line assembly
}

#pragma code


//************************************************
//* Function: isr_low_direct                     *
//*   - Direct execution to the actual           *
//*     low-priority interrupt code.             *
//************************************************
#pragma code isrlowcode = 0x0000018

void isr_low_direct(void)
{  
   //***EXERCISE 2 CODE HERE***
                        										
     _asm 		//begin in-line assembly
     goto isr_low	//go to isr_high function
     _endasm		//end in-line assembly
}
#pragma code


//************************************************
//* Function: isr_high(void)                     *     
//*   High priority interrupt will               *
//************************************************
void isr_high(void)
{
 
}									


//************************************************
//* Function: isr_low(void)                      *
//*   Low priority interrupt will                *
//*   - Check cause of interrupt and             *
//************************************************
void isr_low(void)
{
 
}				


